/**
 * Mysql diver instance.
 * @memberof module:pon-db-sqlite-driver/lib
 * @function resource
 * @param {object} db - SQLite database instance.
 * @param {string} name - Resource name.
 * @param {object} options - Optional settings.
 * @returns {object} Resource instance.
 */

"use strict";

var ponDbDriver = require('pon-db-driver'),
    ponSql = require('pon-sql'),
    async = require('async');

/** @lends resource */
function resource(db, tableName, options) {
    options = options || {};
    var sql = ponSql(tableName, {
        idKey: options.idKey
    });

    function run(sql, callback) {
        db.run(sql, function (err, result) {
            callback(err, format(tableName, result));
        });
    }

    function all(sql, callback) {
        db.all(sql, function (err, result) {
            callback(err, format(tableName, result));
        });
    }

    function seq(tableName, callback) {
        var sql = 'select seq from sqlite_sequence where name="' + tableName + '"';
        all(sql, function (err, data) {
            callback(err, data && data.shift()['seq']);
        })
    }

    var resource = ponDbDriver.resource({
        create: function create(data, callback) {
            run(sql.create(data), function (err) {
                if (err) {
                    callback(err);
                    return;
                }
                async.waterfall([
                    function (callback) {
                        seq(tableName, callback);
                    },
                    function (seq, callback) {
                        resource.one(seq, callback);
                    },
                    function (data, callback) {
                        callback(null, {
                            success: true,
                            data: data
                        });
                    }
                ], callback);
            });
        },
        update: function update(id, data, callback) {
            async.waterfall([
                function (callback) {
                    run(sql.update(id, data), callback);
                },
                function (result, callback) {
                    resource.one(id, callback);
                },
                function (data, callback) {
                    callback(null, {
                        success: true,
                        data: data
                    });
                }
            ], callback);
        },
        updateBulk: function updateBulk(condition, data, callback) {
            async.waterfall([
                function (callback) {
                    run(sql.updateBulk(condition, data), callback);
                },
                function (result, callback) {
                    resource.list({condition: condition}, callback);
                },
                function (data, callback) {
                    callback(null, {
                        success: true,
                        data: data
                    });
                }
            ], callback);
        },
        destroy: function destroy(id, callback) {
            async.waterfall([
                function (callback) {
                    resource.one(id, callback);
                },
                function (data, callback) {
                    run(sql.destroy(id), function done(err) {
                        callback(err, {
                            success: true,
                            data: data
                        });
                    });
                }
            ], callback);
        },
        destroyBulk: function destroyBulk(condition, callback) {
            async.waterfall([
                function (callback) {
                    resource.list({condition: condition}, callback);
                },
                function (data, callback) {
                    run(sql.destroyBulk(condition), function done(err) {
                        callback(err, {
                            success: true,
                            data: data
                        });
                    });
                }
            ], callback);
        },
        one: function one(id, callback) {
            all(sql.one(id), function done(err, result) {
                callback(err, result && result.shift());
            });
        },
        list: function list(config, callback) {
            all(sql.list(config), function done(err, result) {
                callback(err, result);
            });
        },
        count: function count(conifg, callback) {
            all(sql.count(conifg), function done(err, result) {
                callback(err, result && result.shift()['count']);
            });
        },
        define: function (model, callback) {
            var sql = model.toSqliteSql();
            all(sql, callback);
        },
        drop: function (callback) {
            var sql = 'DROP TABLE IF EXISTS ' + tableName;
            all(sql, callback);
        }
    });
    return resource;
}

function format(tableName, data) {
    if (!data) {
        return;
    }
    if (Array.isArray(data)) {
        return data.map(function (data) {
            return format(tableName, data);
        });
    }
    var formatted = {};
    Object.keys(data).forEach(function (key) {
        var val = data[key],
            names = key.split(/\./).filter(function (name) {
                return !!name;
            });
        if (names.length == 1) {
            formatted[key.replace(/^\./, '')] = data[key];
            return;
        }
        var namespace = names[0] && names[0].toLowerCase(),
            col = names[1] && names[1].toLowerCase();
        var hasNameSpace = namespace && (namespace != tableName.toLowerCase());
        if (hasNameSpace) {
            formatted[namespace] = formatted[namespace] || {};
            formatted[namespace][col] = val;
        } else {
            formatted[col] = val;
        }
    });
    return formatted;
}

resource.format = format;
module.exports = resource;