/**
 * Test for driver.
 * Runs with mocha.
 */

"use strict";

var assert = require('assert'),
    setupTestDb = require('../ci/helpers/setup_test_db'),
    async = require('async'),
    mkdirp = require('mkdirp'),
    path = require('path'),
    driver = require('../lib/driver');

var dbFilename = __dirname + '/../tmp/testing-driver.db';
describe('driver', function () {
    before(function (done) {
        mkdirp.sync(path.dirname(dbFilename));
        setupTestDb(dbFilename, function () {
            done();
        });
    });
    after(function (done) {
        process.nextTick(function () {
            done();
        }); //Wait for IO
    });

    it('Crud with resource.', function (done) {
        var connected = driver(dbFilename);
        connected.connect(function () {
            var resource = connected.resource('employees', {
                idKey: 'employee_id'
            });
            var created;
            async.series([
                function (callback) {
                    resource.create({
                        last_name: 'foo',
                        first_name: 'bar',
                        hire_date: new Date()
                    }, function (err, result) {
                        assert.ifError(err);
                        created = result.data;
                        assert.equal(result.success, true);
                        assert.equal(result.data.last_name, 'foo');
                        callback(err);
                    });
                },
                function (callback) {
                    resource.one(created.employee_id, function (err, data) {
                        assert.ifError(err);
                        assert.equal(data.last_name, 'foo');
                        callback(err);
                    });
                },
                function (callback) {
                    resource.update(created.employee_id, {last_name: "baz"}, function (err, result) {
                        assert.ifError(err);
                        assert.equal(result.data.last_name, "baz");
                        callback(err);
                    });
                },
                function (callback) {
                    resource.updateBulk({employee_id: [created.employee_id]}, {last_name: "baz2"}, function (err, result) {
                        assert.ifError(err);
                        assert.equal(result.data[0].last_name, "baz2");
                        callback(err);
                    });
                },
                function (callback) {
                    resource.list({}, function (err, result) {
                        assert.ifError(err);
                        assert.ok(result.length > 0);
                        callback(err);
                    });
                },
                function (callback) {
                    resource.count({}, function (err, count) {
                        assert.ifError(err);
                        assert.ok(count > 0);
                        callback(err);
                    });
                },
                function (callback) {
                    resource.destroy(created.employee_id, function (err, result) {
                        assert.ifError(err);
                        assert.equal(result.data.last_name, "baz2");
                        resource.one(created.employee_id, function (err, data) {
                            assert.ifError(err);
                            assert.ifError(data);
                            callback(err);
                        });
                    });
                },
                function (callback) {
                    resource.destroyBulk({}, function (err, result) {
                        assert.ok(Array.isArray(result.data));
                        assert.ifError(err);
                        resource.one(created.employee_id, function (err, data) {
                            assert.ifError(err);
                            assert.ifError(data);
                            callback(err);
                        });
                    });
                },
                function (callback) {
                    resource.drop(function (err) {
                        callback(err);
                    });
                }
            ], function (err) {
                assert.ifError(err);
                connected.disconnect(function () {
                    done();
                });
            });
        });

    });


});
